/*  ***** BEGIN LICENSE BLOCK *****
 *  Version: RPPL 1.0/GPL 2.0/LGPL 2.1/MPL 1.1/CDDL 1.0
 *  
 *  
 *  The contents of this file are subject to the Roland Pihlakas Public
 *  License Version 1.0 (the "License"); you may not use this file except
 *  in compliance with the License. You may obtain a copy of the License
 *  at http://roland.pri.ee/RPPL/
 *  
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *  
 *  The Original Code is: Modelling of Natural Thinking.
 *  
 *  The Initial Developer of the Original Code is: Roland Pihlakas.
 *  
 *  Portions created by: _____________________________ 
 *  are Copyright (C): _____________________________. 
 *  All Rights Reserved.
 *  Contributor(s): ______________________________________.
 *  
 *  Alternatively, the contents of this file may be used under the terms of 
 *  either the GNU General Public License Version 2 or later (the "GPL"), or 
 *  the GNU Lesser General Public License Version 2.1 or later (the "LGPL"), or 
 *  the Mozilla Public Licence Version 1.1 or later (the "MPL"), or the Common 
 *  Development and Distribution License Version 1.0 or later (the "CDDL"), in 
 *  which case the provisions of the MPL, the GPL, the LGPL or the CDDL are 
 *  applicable instead of those above. If you wish to allow use of your version 
 *  of this file only under the terms of the of either the MPL, the GPL, 
 *  the LGPL or the CDDL, and not to allow others to use your version of this 
 *  file under the RPPL, indicate your decision by deleting the provisions 
 *  above and replace them with the notice and other provisions required by 
 *  the MPL, the GPL, the LGPL or the CDDL. If you do not delete the provisions 
 *  above, a recipient may use your version of this file under the terms of any 
 *  one of the RPPL, the MPL, the GPL, the LGPL or the CDDL.
 *
 *  ***** END LICENSE BLOCK ***** */

//######################################################################################################
//
//######################################################################################################



	
public:

	TasandidController_Server_base()
	{
		
		char datestr[512];
		char filename[512];
		testseries_logger.getDateStr(datestr, true);				///true = for_filename
		sprintf(filename, "testseries %s.txt", datestr);
		testseries_logger.init(filename, LOG_APPEND);
	
	
		action_counter1 = 0;
		action_counter2 = 0;
		reinforcement_is_scheduled = false;
		ratio_tick_is_scheduled = false;
		ratio_ticks = 0;
	
	
	
	
		//net = NULL;
		step = -1;
		step_complete = true;
		steps_occured = true;
		

#if USE_OPERANT_NET				   ///operant net is able and is better of use when always learning (when no-one "teaches", it learns at least avoidance associations / punishments, but potentially reinforcements for some new situations too)
		prevmode = MODE_PREDICT;					
		mode = MODE_LEARN;
#else		
		prevmode = MODE_LEARN;					
		mode = MODE_PREDICT;
#endif


		MAX_NUM_DISPLAY_CHARTS = MAX_NUM_DISPLAY_CHARTS_CONST;



		start_simulationstep = -1;
		
		buttonclick_pause_counter = 0;
		
		lastkeypress = -1;
		
		
		
		operant_preferred_value = 0;	//cell_value(0);
		operant_actual_value = initial_operant_actual_value;	//cell_value(0);
		operant_actual_value__prev = initial_operant_actual_value;

		turnpercentagec1 = 0.5;
		turnpercentagec2 = 0.75;



		discriminant1 = 0;
		discriminant2 = 0;
		discriminant3 = 0;
		
		interval_measure.init();
		
		
		
		action_complete = true;
		
		
		
		outputs.init();
		assigned_outputs.init();
		
		
		initscreen();
	};
	
	~TasandidController_Server_base()				//roland: called on reset and program close
	{
		using namespace curses;
	
		net_exist_Semaphore->lock();				  ///must be outside of DEBUG_SEMAPHORE_LOCK because debug_table_thread_worker calls this outside
		DEBUG_SEMAPHORE_LOCK;						  ///taking multiple semaphores must always be done in the SAME order as they are released!
		
		
		
		if (win != NULL)
		{
			delwin(win);
			endwin();
			
			delscreen(SP);
			
			win = NULL;
		}
		
	
		if (net)
		{	
			//save_net();
		
			delete net;	
			//net->deinit();
			net = NULL;
		}
		
		
		outputs.deinit();
		assigned_outputs.deinit();

		
		
		net_exist_Semaphore->unlock(); 		
		DEBUG_SEMAPHORE_UNLOCK;					 ///taking multiple semaphores must always be done in the SAME order as they are released!
		
	};

	///########################################################################

	void init()
	{	
		
		action_counter1 = 0;
		action_counter2 = 0;
		
		
				
		if (net)
		{	
			
			net_exist_Semaphore->lock();
			DEBUG_SEMAPHORE_LOCK;					  ///taking multiple semaphores must always be done in the SAME order as they are released!
			
			//save_net();
			
			delete net;	
			//net->deinit();
			net = NULL;
			net_exist_Semaphore->unlock();
		}
		else
		{
			DEBUG_SEMAPHORE_LOCK;
		}
		
	
///#if !USE_OPERANT_NET				
		status = WAIT_FOR_START;
		action_complete = true;
///#else
///		status = STOP;
///#endif		
		turn_counter = 0;	
		turn_recovery_counter = 0;
		step = -1;
		
		interval_measure.reset();
		
		 
		 
#if USE_NET	
		
		init_net();		 
		
		init_statuses();
		
#endif	
		DEBUG_SEMAPHORE_UNLOCK;
	};

///########################################################################
///
///########################################################################

protected:

	void save_net()
	{
		net->save();
	};
	
///########################################################################
	
	void load_net()
	{
	
		action_complete = true;
		action_counter1 = 0;
		action_counter2 = 0;
		reinforcement_is_scheduled = false;
		ratio_tick_is_scheduled = false;
		ratio_ticks = 0;
		
		//operant_preferred_value = 0;	//cell_value(0);
		//operant_actual_value = initial_operant_actual_value;	//cell_value(0);
		//operant_actual_value__prev = initial_operant_actual_value;
		
		
	
	
		DEBUG_SEMAPHORE_LOCK;
	
		if (net == NULL)
		{
			//net = &echolines_net_instance;	//
			//net->init();
			net = new tasandid_server_net_type();	
			
		}


		net->set_num_echoline_outputcells(num_echoline_outputcells, 0);
		net->set_num_echoline_outputcells(num_echoline_outputcells2, 1);
		bool rval = net->load(NULL, /*empty_echolines = */ /*TEST_LINEFOLLOWER*/ EMPTY_ECHOLINES_ON_LOAD);
		
		
		if (!rval)			//net was not loaded
		{
			net->set_num_inputs(NUM_NET_BASE_INPUTS);	
			net->set_num_outputs(NUM_NET_BASE_OUTPUTS);
#if USE_OPERANT_NET					
			net->set_num_operants(NUM_NET_OPERANTS);		///right now we use only two operants - joone kalle
											///and whether we are on line or not
#endif		
	
			net->set_num_echoline_outputcells(num_echoline_outputcells, 0);	
			net->set_num_echoline_outputcells(num_echoline_outputcells2, 1);	
		}
		else
		{		
		#if TEST_LINEFOLLOWER
			net->reset_io();
			net->empty_echolines();
			
		#else
			
			#if USE_OPERANT_NET		
			
				//operant_preferred_value = net->get_operant_prev_preferred_atindex(0);	
				operant_actual_value			= net->get_operant_prev_actual_atindex(0);
				operant_actual_value__prev		= operant_actual_value;
					
				
				IFC(!(TEST_LINEFOLLOWER || !USE_DISCRIMINANT_NET))
				{
					discriminant1 = net->get_input_prev_atindex(0);		//values_in.set_item_atindex(0, TA(discriminant1));
					IFC(NUM_NET_BASE_INPUTS	> 1)
						discriminant2 = net->get_input_prev_atindex(1);		//values_in.set_item_atindex(1, TA(discriminant2));
					MYASSERT(NUM_NET_BASE_INPUTS <= 2);
				}
			
			#endif

		#endif
			
			net->set_num_echoline_outputcells(num_echoline_outputcells, 0);				
			net->set_num_echoline_outputcells(num_echoline_outputcells2, 1);
			
			
							
		}
		
		
		
		
		steps_occured = true;					//lets update console
		
		
		DEBUG_SEMAPHORE_UNLOCK;
	};

///########################################################################
///
///########################################################################

public:

	void init_net(bool dontlock = FALSE)
	{
		
		
		
		if (!dontlock)
			net_exist_Semaphore->lock();
		
		DEBUG_SEMAPHORE_LOCK;						///taking multiple semaphores must always be done in the SAME order as they are released!
		
		
		
		//if (net != NULL)
		//	delete net;
		if (net == NULL)
		{
			//net = new ai::assoc_net_1_with_echolines();
		
			load_net();	
		}
		
		steps_occured = true;					//lets update console
		
		
		
		//net->reset_io();
		//net->empty_echolines(); 
		
		
		if (!dontlock)
			net_exist_Semaphore->unlock();
		
		
		DEBUG_SEMAPHORE_UNLOCK;					 ///taking multiple semaphores must always be done in the SAME order as they are released!
	};

	///########################################################################

